@@ -20,21 +20,23 @@ end
 
 def act_install_as_ruby_extension(action, group, kz)
   statusbar_timeout = 3 * 1000
-  statusbar_id = kz.statusbar.get_context_id(action.name)
+  name = action.name
+  uri = nil
   uri = kz.mouse_event_info.link if kz.mouse_event_info
-  uri ||= kz.current_page.location
+  uri = kz.current_page.location if uri.nil? and kz.current_page
+  return if uri.nil?
   filename = extract_extension_filename_from_uri(uri)
   FileUtils.mkdir_p(File.dirname(filename))
 
   dl = Kz::Downloader.new(uri, filename)
   dl.signal_connect("completed") do
     Kz::Actions.install_action(kz, filename)
-    kz.statusbar.push(statusbar_id, _("installed Ruby extension successfully"))
-    Kz.pop_statusbar(kz, statusbar_id, statusbar_timeout)
+    kz.statusbar.set_text(_("installed Ruby extension successfully"), name)
+    Kz.pop_statusbar(kz, name, statusbar_timeout)
   end
   dl.signal_connect("error") do
-    kz.statusbar.push(statusbar_id, _("failed Ruby extension install"))
-    Kz.pop_statusbar(kz, statusbar_id, statusbar_timeout)
+    kz.statusbar.set_text(_("failed Ruby extension install"), name)
+    Kz.pop_statusbar(kz, name, statusbar_timeout)
   end
   Kz::DownloaderGroup.add(dl)
   dl.to_file
