@@ -13,7 +13,7 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
- *  You should have received a copyED of the GNU General Public License
+ *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
@@ -27,6 +27,13 @@
 #define RVAL2KZBOOKMARK(obj) (KZ_BOOKMARK(RVAL2GOBJ(obj)))
 #define RVAL2KZNAVI(obj) (KZ_NAVI(RVAL2GOBJ(obj)))
 #define RVAL2KZNAVLINK(obj) (RVAL2GENUM(obj, KZ_TYPE_EMBED_NAV_LINK))
+#define RVAL2KZRF(obj) (RVAL2GENUM(obj, KZ_TYPE_EMBED_RELOAD_FLAG))
+
+static VALUE
+rb_kz_embed_get_window(VALUE self)
+{
+    return GOBJ2RVAL(kz_window_get_from_tab(RVAL2GOBJ(self)));
+}
 
 static VALUE
 rb_kz_embed_load_url(VALUE self, VALUE url)
@@ -266,7 +273,7 @@ rb_kz_embed_shistory_get_nth(VALUE self, VALUE nth, VALUE is_relative)
 static VALUE
 rb_kz_embed_reload(VALUE self, VALUE flags)
 {
-    kz_embed_reload(_SELF(self), NUM2INT(flags));
+    kz_embed_reload(_SELF(self), RVAL2KZRF(flags));
     return self;
 }
 
@@ -620,6 +627,13 @@ Init_kz_rb_embed(VALUE mKz)
     cKzEmbed = G_DEF_CLASS(KZ_TYPE_EMBED, "Embed", mKz);
     cKzMozEmbed = G_DEF_CLASS(KZ_TYPE_MOZ_EMBED, "MozEmbed", mKz);
 
+    G_DEF_CLASS(KZ_TYPE_EMBED_LOAD_FLAG, "EmbedLoadFlag", mKz);
+    G_DEF_CLASS(KZ_TYPE_EMBED_RELOAD_FLAG, "EmbedReloadFlag", mKz);
+    G_DEF_CLASS(KZ_TYPE_EMBED_GO_ACTION, "EmbedGoAction", mKz);
+    G_DEF_CLASS(KZ_TYPE_EMBED_NAV_LINK, "EmbedNavLink", mKz);
+
+    rb_define_method(cKzEmbed, "window", rb_kz_embed_get_window, 0);
+
     rb_define_method(cKzEmbed, "load_url", rb_kz_embed_load_url, 1);
     rb_define_method(cKzEmbed, "view_source",
                      rb_kz_embed_view_source, 1);
