@@ -44,7 +44,7 @@ static VALUE
 rb_kz_window_get_window_list(VALUE self)
 {
     GList *list;
-    list = kz_window_get_window_list();
+    list = (GList *)kz_window_get_window_list();
     return GLIST2ARY(list);
 }
 
