@@ -19,6 +19,7 @@
  */
 
 #include "kz-actions-dynamic.h"
+#include "kz-smart-bookmark.h"
 
 #include <string.h>
 #include <glib/gi18n.h>
@@ -28,6 +29,7 @@
 #include "kz-navi.h"
 
 #define KZ_ACTIONS_DYNAMIC_COPY_IN_USER_FORMAT_KEY "KzActionsDynamic::CopyInUserFormat"
+#define KZ_ACTIONS_DYNAMIC_SMART_LIST_KEY "KzActionsDynamic::SmartList"
 
 
 static void
@@ -59,7 +61,7 @@ cb_copy_in_user_format_menuitem_activate (GtkWidget *menuitem, KzTabLabel *kztab
 	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
 
 	kzembed = kztab->kzembed;
-	 if (!KZ_IS_EMBED(kzembed)) return;
+	if (!KZ_IS_EMBED(kzembed)) return;
 
 	text = kz_embed_get_selection_string(kzembed);
 	if (!text) text = g_strdup("");
@@ -107,13 +109,27 @@ cb_copy_in_user_format_menuitem_activate (GtkWidget *menuitem, KzTabLabel *kztab
 			else if(splited[i][0] == 't')
 			{
 				/* %t% */
-				temp[i] = (gchar*)kz_embed_get_title(kzembed);
+				const KzEmbedEventMouse *event;
+
+				event = kz_window_get_mouse_event_info(kztab->kz);
+				if (event && event->cinfo.linktext)
+					temp[i] = event->cinfo.linktext;
+				else
+					temp[i] = (gchar *)kz_embed_get_title(kzembed);
+
 				st = 2;
 			}
 			else if(splited[i][0] == 'u')
 			{
 				/* %u% */
-				temp[i] = (gchar*)kz_embed_get_location(kzembed);
+				const KzEmbedEventMouse *event;
+
+				event = kz_window_get_mouse_event_info(kztab->kz);
+				if (event && event->cinfo.link)
+					temp[i] = event->cinfo.link;
+				else
+					temp[i] = (gchar *)kz_embed_get_location(kzembed);
+
 				st = 2;
 			}
 			else if(splited[i][0] == 's')
@@ -218,11 +234,12 @@ cb_add_feed_bookmark_menuitem_activate (GtkWidget *widget, KzNavi *navi)
 }
 
 void
-kz_actions_dynamic_append_add_feed_bookmark (KzEmbed *kzembed,
+kz_actions_dynamic_append_add_feed_bookmark (KzWindow *kz,
 					     GtkMenuItem *menu)
 {
 	GtkWidget *add_feed_bookmark_submenu = NULL;
 	GtkWidget *menuitem;
+	GtkWidget *kzembed;
 	GList *nav_link, *nav_links;
 	KzNavi *navi;
 	gchar *title;
@@ -233,7 +250,10 @@ kz_actions_dynamic_append_add_feed_bookmark (KzEmbed *kzembed,
 		gtk_menu_item_remove_submenu(GTK_MENU_ITEM(menu));
 	}
 
-	nav_links = kz_embed_get_nav_links(kzembed, KZ_EMBED_LINK_RSS);
+	kzembed = KZ_WINDOW_CURRENT_PAGE(kz);
+	if (!KZ_IS_EMBED(kzembed)) return;
+
+	nav_links = kz_embed_get_nav_links(KZ_EMBED(kzembed), KZ_EMBED_LINK_RSS);
 
 	if (!nav_links) return;
 	
@@ -263,3 +283,104 @@ kz_actions_dynamic_append_add_feed_bookmark (KzEmbed *kzembed,
 		gtk_widget_show(menuitem);
 	}
 }
+
+static void
+cb_open_smart_bookmark_menuitem_activate (GtkWidget *widget, KzWindow *kz)
+{
+	KzSmartBookmark *smart_bookmark;
+	GtkWidget *embed;
+	const gchar *text;
+	gchar *smart_uri;
+	gboolean new_tab;
+
+	smart_bookmark = g_object_get_data(G_OBJECT(widget),
+					   KZ_ACTIONS_DYNAMIC_SMART_LIST_KEY);
+
+	g_return_if_fail(KZ_IS_SMART_BOOKMARK(smart_bookmark));
+
+	embed = KZ_WINDOW_CURRENT_PAGE(kz);
+	text = kz_embed_get_selection_string(KZ_EMBED(embed));
+	smart_uri = kz_smart_bookmark_get_smart_uri(smart_bookmark, text);
+
+	KZ_CONF_GET("Global", "entry_open_in_new_tab", new_tab, BOOL);
+	if (new_tab)
+		kz_window_open_new_tab(kz, smart_uri);
+	else
+		kz_window_load_url(kz, smart_uri);
+
+	g_free(smart_uri);
+}
+
+void
+kz_actions_dynamic_append_open_smart_bookmark_menuitem (KzWindow *kz,
+							GtkMenuItem *menu)
+{
+	GList *children, *node;
+	GtkWidget *smart_bookmark_submenu = NULL;
+	const gchar *title;
+	KzBookmark *bookmark = kz_bookmarks->smarts;
+	KzBookmark *child, *smart;
+
+	smart_bookmark_submenu = gtk_menu_item_get_submenu(GTK_MENU_ITEM(menu));
+	if (smart_bookmark_submenu)
+	{
+		gtk_menu_item_remove_submenu(GTK_MENU_ITEM(menu));
+	}
+
+	smart_bookmark_submenu = gtk_menu_new();
+	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu), smart_bookmark_submenu);
+	gtk_widget_show(smart_bookmark_submenu);
+
+	children = kz_bookmark_get_children(bookmark);
+	for (node = children; node; node = g_list_next(node))
+	{
+		GList *sub_children, *sub_node;
+		GtkWidget *sub_folder, *sub_bookmark;
+		GtkWidget *menuitem;
+
+		child = node->data;
+		if (!kz_bookmark_is_folder(child))
+			continue;
+
+		title = kz_bookmark_get_title(child);
+		if (title && g_str_has_prefix(title, "LocationEntry"))
+			continue;
+
+		menuitem = gtk_menu_item_new_with_label(title);
+		gtk_menu_shell_append(GTK_MENU_SHELL(smart_bookmark_submenu),
+				      menuitem);
+
+		sub_folder = gtk_menu_new();
+		gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuitem),
+					  sub_folder);
+		gtk_widget_show(sub_folder);
+		gtk_widget_show(menuitem);
+
+		sub_children = kz_bookmark_get_children(child);
+
+		for (sub_node = sub_children;
+		     sub_node;
+		     sub_node = g_list_next(sub_node))
+		{
+			const gchar *bookmark_title;
+
+			smart = sub_node->data;
+			bookmark_title = kz_bookmark_get_title(smart);
+
+			sub_bookmark =
+				gtk_menu_item_new_with_label(bookmark_title);
+			g_object_set_data (G_OBJECT(sub_bookmark),
+					   KZ_ACTIONS_DYNAMIC_SMART_LIST_KEY,
+					   smart);
+
+			gtk_menu_shell_append(GTK_MENU_SHELL(sub_folder),
+					      sub_bookmark);
+			g_signal_connect(sub_bookmark, "activate",
+					 G_CALLBACK(cb_open_smart_bookmark_menuitem_activate),
+					 kz);
+			gtk_widget_show(sub_bookmark);
+		}
+		g_list_free(sub_children);
+	}
+	g_list_free(children);
+}
