@@ -58,6 +58,7 @@
 #include "eggregex.h"
 #include "kz-proxy-folder.h"
 #include "kz-icons.h"
+#include "kz-statusbar.h"
 
 #include <time.h>
 
@@ -629,7 +630,7 @@ act_find (GtkAction *action, KzWindow *kz)
 	g_return_if_fail(GTK_IS_ACTION(action));
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 
-	gtk_widget_grab_focus(kz->find_area);
+	kz_statusbar_set_focus_to_find_area(KZ_STATUSBAR(kz->statusbar));
 }
 
 static void
@@ -1420,7 +1421,8 @@ act_save_session (GtkAction *action, KzWindow *kz)
 static void
 act_restore_session (GtkAction *action, KzWindow *kz)
 {
-	GList *list, *node;
+	GList *list;
+	const GList *node;
 	gchar *current_session;
 	gint i, num, pos = 0;
 
@@ -1429,7 +1431,7 @@ act_restore_session (GtkAction *action, KzWindow *kz)
 	if (is_restoring_session) return;
 #if 1
 	{
-	GList *window_list, *window_node;
+	const GList *window_list, *window_node;
 	KzBookmark *tmp_bookmark;
 
 	/* reload from current session file */
@@ -1900,7 +1902,7 @@ act_view_fullscreen (GtkAction *action, KzWindow *kz)
 	{
 		gtk_window_fullscreen(GTK_WINDOW(kz));
 		gtk_widget_hide(kz->bookmark_bars_area);
-		gtk_widget_hide(kz->statusbar_area);
+		gtk_widget_hide(kz->statusbar);
 		gtk_widget_hide(kz->menu_box);
 		gtk_notebook_set_show_tabs(GTK_NOTEBOOK(kz->notebook), FALSE);
 	
@@ -1911,7 +1913,7 @@ act_view_fullscreen (GtkAction *action, KzWindow *kz)
 		gboolean toggle_active;
 
 		gtk_window_unfullscreen(GTK_WINDOW(kz));
-		gtk_widget_show(kz->statusbar_area);
+		gtk_widget_show(kz->statusbar);
 		gtk_widget_show(kz->menu_box);
 		gtk_notebook_set_show_tabs(GTK_NOTEBOOK(kz->notebook), TRUE);
 		
