@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-bookmark-item.c,v 1.10 2005/04/22 04:44:38 ikezoe Exp $
+ *  $Id: kz-bookmark-item.c,v 1.20 2005/07/21 14:43:28 ikezoe Exp $
  */
 
 #include "kz-bookmark-item.h"
@@ -30,11 +30,12 @@
 #include "kazehakase.h"
 #include "kz-bookmark-file.h"
 #include "kz-bookmark-menu.h"
+#include "kz-bookmark-menu-item.h"
+#include "kz-bookmark-folder-menu-item.h"
 #include "kz-bookmark-editor.h"
 #include "kz-icons.h"
 #include "kz-actions.h"
 #include "gtk-utils.h"
-#include "kz-input-event-box.h"
 
 enum {
 	PROP_0,
@@ -44,13 +45,23 @@ enum {
 
 enum {
 	TARGET_KAZEHAKASE_BOOKMARKS,
+	TARGET_NETSCAPE_URL,
+	TARGET_TEXT_URI_LIST
 };
 
 static GtkTargetEntry dnd_types[] = {
 	{"_KAZEHAKASE_BOOKMARKS", 0, TARGET_KAZEHAKASE_BOOKMARKS},
 };
 
+static GtkTargetEntry url_drag_types [] = 
+{
+	{"_KAZEHAKASE_BOOKMARKS", 0, TARGET_KAZEHAKASE_BOOKMARKS},
+        { "_NETSCAPE_URL",        0, TARGET_NETSCAPE_URL},
+	{ "text/uri-list",        0, TARGET_TEXT_URI_LIST}
+};
+
 static const gint dnd_types_num = G_N_ELEMENTS(dnd_types);
+static const gint n_url_drag_types = G_N_ELEMENTS(url_drag_types);
 
 static void     kz_bookmark_item_class_init       (KzBookmarkItemClass *klass);
 static void     kz_bookmark_item_init             (KzBookmarkItem *menu);
@@ -68,6 +79,9 @@ static void     kz_bookmark_item_get_property     (GObject        *object,
 						   guint           prop_id,
 						   GValue         *value,
 						   GParamSpec     *pspec);
+/* GtkToolItem class methods */
+static gboolean kz_bookmark_item_create_menu_proxy (GtkToolItem     *item);
+
 /* widget class methods */
 static void     kz_bookmark_item_drag_data_get    (GtkWidget        *widget,
 						   GdkDragContext   *context,
@@ -76,6 +90,8 @@ static void     kz_bookmark_item_drag_data_get    (GtkWidget        *widget,
 						   guint             time);
 static void     kz_bookmark_item_drag_begin       (GtkWidget        *widget,
 						   GdkDragContext   *context);
+static void     kz_bookmark_item_drag_end         (GtkWidget        *widget,
+						   GdkDragContext   *context);
 
 /* private methods */
 static void     kz_bookmark_item_reset_submenu    (KzBookmarkItem *menu);
@@ -88,8 +104,6 @@ static void     cb_bookmark_load_stop             (KzBookmark *bookmark,
 static void     cb_bookmark_load_error            (KzBookmark *bookmark,
 						   const gchar *error,
 						   KzBookmarkItem *menu);
-static void     bookmark_weak_ref                 (gpointer data,
-						   GObject *obj);
 
 /* callbacks for child widgets */
 static gboolean cb_bookmark_menu_release          (GtkWidget        *widget,
@@ -97,31 +111,39 @@ static gboolean cb_bookmark_menu_release          (GtkWidget        *widget,
 						   KzBookmarkItem   *menu);
 static void     cb_update_button_clicked          (GtkWidget        *widget,
 						   KzBookmarkItem   *menu);
-static void     cb_menuitem_destroy               (GtkWidget        *widget,
-						   KzBookmark       *item);
 
-/* manage life cycle of both menuitems and bookmarks */
-static void     connect_observe_signals           (GtkWidget *widget,
-						   KzBookmark *item);
+static void	cb_notify_title 		  (GObject *object, GParamSpec *pspec,
+						   KzBookmarkItem *menu);
+
+static void     cb_drag_data_received             (GtkWidget        *widget,
+						   GdkDragContext   *context,
+						   gint              x,
+						   gint              y,
+			                           GtkSelectionData *selection_data,
+						   guint             info,
+						   guint             time,
+						   gpointer          data);
 
 
-static GtkHBoxClass *parent_class = NULL;
+static GtkToolItemClass *parent_class = NULL;
 
 
 KZ_OBJECT_GET_TYPE(kz_bookmark_item, "KzBookmarkItem", KzBookmarkItem,
 		   kz_bookmark_item_class_init, kz_bookmark_item_init,
-		   GTK_TYPE_HBOX)
+		   GTK_TYPE_TOOL_ITEM)
 
 static void
 kz_bookmark_item_class_init (KzBookmarkItemClass *klass)
 {
 	GObjectClass *gobject_class;
 	GtkWidgetClass *widget_class;
+	GtkToolItemClass *tool_item_class;
 
 	parent_class = g_type_class_peek_parent (klass);
 
-	gobject_class = (GObjectClass *) klass;
-	widget_class  = (GtkWidgetClass *) klass;
+	gobject_class   = (GObjectClass *) klass;
+	widget_class    = (GtkWidgetClass *) klass;
+	tool_item_class = (GtkToolItemClass *) klass;
 
 	/* GObject signals */
 	gobject_class->dispose      = kz_bookmark_item_dispose;
@@ -131,6 +153,9 @@ kz_bookmark_item_class_init (KzBookmarkItemClass *klass)
 
 	widget_class->drag_data_get = kz_bookmark_item_drag_data_get;
 	widget_class->drag_begin    = kz_bookmark_item_drag_begin;
+	widget_class->drag_end      = kz_bookmark_item_drag_end;
+
+  	tool_item_class->create_menu_proxy = kz_bookmark_item_create_menu_proxy;
 
 	g_object_class_install_property
 		(gobject_class,
@@ -169,9 +194,9 @@ kz_bookmark_item_constructor (GType                  type,
 			      guint                  n_props,
 			      GObjectConstructParam *props)
 {
-	KzBookmarkItem *menu;
-	GtkWidget *button, *image, *eventbox;
-	GtkBox *hbox;
+	KzBookmarkItem *item;
+	GtkWidget *button, *image, *eventbox, *hbox;
+	GtkContainer *container;
 	GtkRcStyle *style;
 	GObject *object;
 	GObjectClass *klass = G_OBJECT_CLASS(parent_class);
@@ -180,35 +205,44 @@ kz_bookmark_item_constructor (GType                  type,
 
 	object = klass->constructor(type, n_props, props);
 
-	hbox = GTK_BOX(object);
-	menu = KZ_BOOKMARK_ITEM(object);
-	is_file = KZ_IS_BOOKMARK_FILE(menu->bookmark);
-	is_folder = kz_bookmark_is_folder(menu->bookmark);
+	container = GTK_CONTAINER(object);
+	hbox = gtk_hbox_new(FALSE, 1);
+	item = KZ_BOOKMARK_ITEM(object);
+	is_file = KZ_IS_BOOKMARK_FILE(item->bookmark);
+	is_folder = kz_bookmark_is_folder(item->bookmark);
 	
-	gtk_container_set_border_width(GTK_CONTAINER(menu), 2);
+	gtk_container_set_border_width(container, 2);
+	gtk_container_add(container, hbox);
+	gtk_widget_show (hbox);
 
 	/* button */
-	menu->update_button = button = gtk_button_new();
+	item->update_button = button = gtk_button_new();
 	gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
-	gtk_box_pack_start(hbox, button, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
 	if (is_file)
 	{
 		g_signal_connect(button, "clicked", 
 				 G_CALLBACK(cb_update_button_clicked),
-				 menu);
-		menu->image = image
+				 item);
+		item->image = image
 			= gtk_image_new_from_stock(KZ_STOCK_GREEN,
 						   KZ_ICON_SIZE_BOOKMARK_MENU);
 	}
 	else
 	{
 		g_signal_connect(button, "button_release_event",
-			 	 G_CALLBACK(cb_bookmark_menu_release), menu);
+			 	 G_CALLBACK(cb_bookmark_menu_release), item);
 		if(is_folder)
 		{
-			menu->image = image
+			item->image = image
 				= gtk_image_new_from_stock(KZ_STOCK_FOLDER,
 							   KZ_ICON_SIZE_BOOKMARK_MENU);
+			gtk_drag_dest_set(GTK_WIDGET(item), 
+					  GTK_DEST_DEFAULT_ALL, 
+	               		          url_drag_types, n_url_drag_types,
+			  		  GDK_ACTION_COPY | GDK_ACTION_LINK | GDK_ACTION_MOVE);
+			g_signal_connect(item, "drag-data-received", 
+					 G_CALLBACK(cb_drag_data_received), NULL);
 		}
 		else
 		{
@@ -216,23 +250,25 @@ kz_bookmark_item_constructor (GType                  type,
 			GtkWidget *favicon = NULL;
 
 			/* favicon */
+			/* kzfav = item->kz->kzfav; */
 			kzfav = kz_favicon_get_instance();
 			favicon	= kz_favicon_get_widget(kzfav,
-							kz_bookmark_get_link(menu->bookmark),
+							kz_bookmark_get_link(item->bookmark),
 							KZ_ICON_SIZE_BOOKMARK_MENU);
+			g_object_unref(kzfav);
 			if (favicon)
 			{
-				menu->image = image = favicon;
+				item->image = image = favicon;
 			}
 			else
 			{
-				menu->image = image
+				item->image = image
 					= gtk_image_new_from_stock(KZ_STOCK_BOOKMARK,
 								   KZ_ICON_SIZE_BOOKMARK_MENU);
 			}
-			g_object_unref(kzfav);
 		}
 	}
+
 	gtk_container_add(GTK_CONTAINER(button), image);
 	gtk_widget_show(image);
 	gtk_widget_show(button);
@@ -243,36 +279,37 @@ kz_bookmark_item_constructor (GType                  type,
 
 	if (is_file)
 	{
-		g_signal_connect(menu->bookmark,
+		g_signal_connect(item->bookmark,
 				 "load_start", 
 				 G_CALLBACK(cb_bookmark_load_start),
-				 menu);
-		g_signal_connect(menu->bookmark,
+				 item);
+		g_signal_connect(item->bookmark,
 				 "load_completed", 
 				 G_CALLBACK(cb_bookmark_load_stop),
-				 menu);
-		g_signal_connect(menu->bookmark,
+				 item);
+		g_signal_connect(item->bookmark,
 				 "error", 
 				 G_CALLBACK(cb_bookmark_load_error),
-				 menu);
+				 item);
 
-		state = kz_bookmark_file_get_state(KZ_BOOKMARK_FILE(menu->bookmark));
+		state = kz_bookmark_file_get_state(KZ_BOOKMARK_FILE(item->bookmark));
 		if (state == KZ_BOOKMARK_FILE_STATE_LOADING)
-			cb_bookmark_load_start(menu->bookmark, menu);
+			cb_bookmark_load_start(item->bookmark, item);
 	}
 	/* label */
-	eventbox = kz_input_event_box_new();
-	menu->eventbox = eventbox;
-	gtk_box_pack_start(hbox, eventbox, FALSE, FALSE, 0);
+	eventbox = gtk_event_box_new();
+	gtk_event_box_set_visible_window(GTK_EVENT_BOX(eventbox), FALSE);
+	item->eventbox = eventbox;
+	gtk_box_pack_start(GTK_BOX(hbox), eventbox, FALSE, FALSE, 0);
 	g_signal_connect(eventbox, "button_release_event",
-			 G_CALLBACK(cb_bookmark_menu_release), menu);
+			 G_CALLBACK(cb_bookmark_menu_release), item);
 	gtk_widget_show (eventbox);
 
-	menu->label = gtk_label_new(kz_bookmark_get_title(menu->bookmark));
-	gtk_container_add(GTK_CONTAINER(eventbox), menu->label);
-	gtk_widget_show (menu->label);
+	item->label = gtk_label_new(kz_bookmark_get_title(item->bookmark));
+	gtk_container_add(GTK_CONTAINER(eventbox), item->label);
+	gtk_widget_show (item->label);
 
-	gtk_drag_source_set(GTK_WIDGET(menu),
+	gtk_drag_source_set(GTK_WIDGET(item),
 			    GDK_BUTTON1_MASK |
 			    GDK_BUTTON2_MASK |
 			    GDK_BUTTON3_MASK,
@@ -281,9 +318,10 @@ kz_bookmark_item_constructor (GType                  type,
 			    GDK_ACTION_ASK  | GDK_ACTION_COPY
 			    | GDK_ACTION_MOVE | GDK_ACTION_LINK);
 	/* submenu */
-	kz_bookmark_item_reset_submenu(menu);
+	kz_bookmark_item_reset_submenu(item);
 
-	connect_observe_signals(GTK_WIDGET(menu), menu->bookmark);
+	g_signal_connect(item->bookmark, "notify::title",
+			 G_CALLBACK(cb_notify_title), item);
 
 	return object;
 }
@@ -316,13 +354,16 @@ kz_bookmark_item_dispose (GObject *object)
 				cb_bookmark_load_error,
 				menu);
 		}
+		g_signal_handlers_disconnect_by_func(menu->bookmark,
+					     G_CALLBACK(cb_notify_title),
+					     menu);
 		g_object_unref(menu->bookmark);
 		menu->bookmark = NULL;
 	}
 
 	if (menu->sub_menu)
 	{
-		g_object_unref(G_OBJECT(menu->sub_menu));
+		gtk_widget_destroy(menu->sub_menu);
 		menu->sub_menu = NULL;
 	}
 
@@ -378,7 +419,29 @@ kz_bookmark_item_get_property (GObject         *object,
 }
 
 
-GtkWidget *
+static gboolean
+kz_bookmark_item_create_menu_proxy (GtkToolItem *item)
+{
+	KzBookmarkItem *bookmark_item = KZ_BOOKMARK_ITEM(item);
+	GtkWidget *proxy;
+
+	if (kz_bookmark_is_folder(bookmark_item->bookmark))
+	{
+		proxy = kz_bookmark_folder_menu_item_new(bookmark_item->kz,
+							 bookmark_item->bookmark);
+	}
+	else
+	{
+		proxy = kz_bookmark_menu_item_new(bookmark_item->kz,
+						  bookmark_item->bookmark);
+	}
+	gtk_tool_item_set_proxy_menu_item(GTK_TOOL_ITEM(item),
+					  "bookmark", proxy);
+  
+	return TRUE;
+}
+
+GtkToolItem *
 kz_bookmark_item_new (KzWindow *kz, KzBookmark *bookmark)
 {
 	KzBookmarkItem *menu;
@@ -391,7 +454,7 @@ kz_bookmark_item_new (KzWindow *kz, KzBookmark *bookmark)
 			   "bookmark",  bookmark,
 			   NULL);
 
-	return GTK_WIDGET(menu);
+	return GTK_TOOL_ITEM(menu);
 }
 
 
@@ -410,10 +473,11 @@ kz_bookmark_item_reset_submenu (KzBookmarkItem *menu)
 
 	submenu = kz_bookmark_menu_create_submenu(menu->bookmark, menu->kz);
 	if (menu->sub_menu)
-		g_object_unref(menu->sub_menu);
+	{
+		gtk_widget_destroy(menu->sub_menu);
+	}
+
 	menu->sub_menu = submenu;
-	g_object_ref(G_OBJECT(menu->sub_menu));
-	gtk_object_sink(GTK_OBJECT(menu->sub_menu));
 }
 
 
@@ -481,20 +545,6 @@ cb_notify_title (GObject *object, GParamSpec *pspec,
 }
 
 
-static void
-bookmark_weak_ref (gpointer data, GObject *obj)
-{
-	KzBookmark *item = KZ_BOOKMARK(obj);
-	GtkWidget *widget = data;
-
-	if (!widget) return;
-
-	g_signal_handlers_disconnect_by_func(widget,
-					     G_CALLBACK(cb_menuitem_destroy),
-					     item);
-}
-
-
 static gboolean
 cb_bookmark_menu_release (GtkWidget *widget, GdkEventButton *event,
 			  KzBookmarkItem *menu)
@@ -566,16 +616,6 @@ cb_update_button_clicked (GtkWidget *widget, KzBookmarkItem *menu)
 }
 
 
-static void
-cb_menuitem_destroy(GtkWidget *widget, KzBookmark *item)
-{
-	g_signal_handlers_disconnect_by_func(item,
-					     G_CALLBACK(cb_notify_title),
-					     widget);
-	g_object_weak_unref(G_OBJECT(item), bookmark_weak_ref, widget);
-}
-
-
 static void
 kz_bookmark_item_drag_data_get (GtkWidget *widget,
 				GdkDragContext *context,
@@ -610,17 +650,133 @@ kz_bookmark_item_drag_begin (GtkWidget *widget, GdkDragContext *context)
 	else
 		stock_id = KZ_STOCK_BOOKMARK;
 
+	gtk_widget_hide(widget);
 	gtk_drag_source_set_icon_stock(widget, stock_id);
 }
 
 
 static void
-connect_observe_signals (GtkWidget *widget, KzBookmark *item)
+kz_bookmark_item_drag_end (GtkWidget *widget, GdkDragContext *context)
 {
-	g_signal_connect(item, "notify::title",
-			 G_CALLBACK(cb_notify_title), widget);
-	g_object_weak_ref(G_OBJECT(item), bookmark_weak_ref, widget);
-
-	g_signal_connect(widget, "destroy",
-			 G_CALLBACK(cb_menuitem_destroy), item);
+	gtk_widget_show(widget);
 }
+
+
+static void
+cb_drag_data_received (GtkWidget        *widget,
+		       GdkDragContext   *context,
+		       gint              x,
+		       gint              y,
+		       GtkSelectionData *selection_data,
+		       guint             info,
+		       guint             time,
+		       gpointer          data)
+{
+	KzBookmarkItem *item;
+	KzBookmark *bookmark, *parent, *file;
+	gchar **strings = NULL; 
+	GtkWidget *src_widget;
+	const gchar *title;
+	gchar *utf8_title = NULL;
+
+	item = KZ_BOOKMARK_ITEM(widget);
+	switch (info)
+	{
+	 case TARGET_KAZEHAKASE_BOOKMARKS:
+		src_widget = gtk_drag_get_source_widget(context);
+		
+		if (!KZ_IS_BOOKMARK_ITEM(src_widget))
+			break;
+
+		bookmark = KZ_BOOKMARK_ITEM(src_widget)->bookmark;
+		g_object_ref(bookmark);
+
+		parent = kz_bookmark_get_parent(bookmark);
+		kz_bookmark_remove(parent, bookmark);
+
+		kz_bookmark_append(item->bookmark, bookmark);
+		if (KZ_IS_BOOKMARK_FILE(item->bookmark))
+			file = item->bookmark;
+		else
+			file = kz_bookmark_get_parent_file(item->bookmark);
+		
+		if (kz_bookmark_file_has_xmlrpc(KZ_BOOKMARK_FILE(file)))
+		{
+			kz_bookmark_file_xmlrpc_insert(KZ_BOOKMARK_FILE(file),
+						       item->bookmark,
+						       NULL,
+						       bookmark);
+		}
+		else
+		{
+			kz_bookmark_file_save(KZ_BOOKMARK_FILE(file));
+		}
+
+		gtk_drag_finish(context, TRUE, TRUE, time);
+	 	break;
+	 case TARGET_NETSCAPE_URL:
+	 case TARGET_TEXT_URI_LIST:
+		if (selection_data->length < 0) return;
+		strings = g_strsplit((const gchar*)selection_data->data, "\n", 2);
+		if (!strings) return;
+		if (strings[1] != NULL)
+		{
+			utf8_title = g_locale_to_utf8(strings[1],
+						      strlen(strings[1]),
+						      NULL,
+						      NULL,
+						      NULL);
+		}
+		if (utf8_title && g_utf8_validate(utf8_title, -1, NULL))
+			title = utf8_title;
+		else 
+			title = _("title");
+		
+		/* FIXME! we should also add normal bookmark. */
+		/* Create New KzBookmark */
+		if (strstr(strings[0], "xml") ||
+		    strstr(strings[0], "rss") ||
+		    strstr(strings[0], "rdf"))
+		{
+			bookmark = KZ_BOOKMARK(kz_bookmark_file_new(strings[0],
+							title,
+							NULL));
+		}
+		else
+		{
+			bookmark = kz_bookmark_new_with_attrs(title,
+							      strings[0],
+							      NULL);
+		}
+		kz_bookmark_append(item->bookmark, bookmark);
+		g_strfreev(strings);
+		if (utf8_title)
+			g_free(utf8_title);
+		g_object_unref(bookmark);
+
+		if (KZ_IS_BOOKMARK_FILE(item->bookmark))
+			file = item->bookmark;
+		else
+			file = kz_bookmark_get_parent_file(item->bookmark);
+		
+		if (kz_bookmark_file_has_xmlrpc(KZ_BOOKMARK_FILE(file)))
+		{
+			kz_bookmark_file_xmlrpc_insert(KZ_BOOKMARK_FILE(file),
+						       item->bookmark,
+						       NULL,
+						       bookmark);
+		}
+		else
+		{
+			kz_bookmark_file_save(KZ_BOOKMARK_FILE(file));
+		}
+	
+		gtk_drag_finish(context, TRUE, FALSE, time);
+		break;
+	 default:
+	 	gtk_drag_finish(context, FALSE, FALSE, time);
+		break;
+	}
+}
+
+
