@@ -23,6 +23,7 @@
 #include <stdio.h>
 #include <gdk/gdkkeysyms.h>
 #include <glib/gi18n.h>
+#include <glib/gstdio.h>
 
 #include "gobject-utils.h"
 #include "kz-bookmark-filter.h"
@@ -391,7 +392,7 @@ kz_links_dialog_save (KzLinksDialog *kzlinks, const gchar *filename)
 	GtkTreeIter iter;
 	gboolean exist;
 
-	fp = fopen(filename, "wt");
+	fp = g_fopen(filename, "wt");
 	if (!fp)
 	{
 		GtkWidget *dialog;
