@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-khtml.c,v 1.19 2005/08/13 09:33:14 kous Exp $
+ *  $Id: kz-khtml.c,v 1.20 2006/01/15 06:29:03 kous Exp $
  */
 
 #include <glib/gi18n.h>
@@ -65,7 +65,7 @@ static gchar       *kz_khtml_ensure_title     (KzEmbed      *kzembed);
 static gchar       *kz_khtml_get_link_message (KzEmbed      *kzembed);
 #endif
 static void         kz_khtml_reload           (KzEmbed      *kzembed,
-					       gint32        flags);
+					       KzEmbedReloadFlag flags);
 static void         kz_khtml_stop_load        (KzEmbed      *kzembed);
 static void         kz_khtml_go_back          (KzEmbed      *kzembed);
 static void         kz_khtml_go_forward       (KzEmbed      *kzembed);
@@ -426,7 +426,7 @@ kz_khtml_ensure_title (KzEmbed *kzembed)
 
 #warning FIXME! use flags
 static void
-kz_khtml_reload (KzEmbed *kzembed, gint32 flags)
+kz_khtml_reload (KzEmbed *kzembed, KzEmbedReloadFlag flags)
 {
 	g_return_if_fail(KZ_IS_KHTML(kzembed));
 
