@@ -114,6 +114,9 @@ struct _KzEmbedIFace {
 						 GList	      **list,
 						 gboolean	selected_only);
 
+	gboolean	(*get_dest_anchors)	(KzEmbed       *kzembed,
+						 GList	      **list);
+
 	void		(*copy_page)		(KzEmbed       *kzembed,
 						 KzEmbed       *dkzembed);
 
@@ -132,7 +135,7 @@ struct _KzEmbedIFace {
 						 gchar	      **aTitle);
 
 	void		(*reload)		(KzEmbed       *kzembed,
-						 gint32		flags);
+						 KzEmbedReloadFlag flags);
 	void		(*stop_load)		(KzEmbed       *kzembed);
 	void		(*go_back)		(KzEmbed       *kzembed);
 	void		(*go_forward)		(KzEmbed       *kzembed);
@@ -325,6 +328,9 @@ gboolean	kz_embed_get_links		(KzEmbed       *kzembed,
 						 GList	      **list,
 						 gboolean	selected_only);
 
+gboolean	kz_embed_get_dest_anchors	(KzEmbed       *kzembed,
+						 GList	      **list);
+
 void		kz_embed_copy_page		(KzEmbed       *kzembed,
 						 KzEmbed       *dkzembed);
 
