@@ -28,38 +28,13 @@
 
 G_BEGIN_DECLS
 
-#define KZ_TYPE_EXT			(kz_ext_get_type ())
-#define KZ_EXT(obj)			(G_TYPE_CHECK_INSTANCE_CAST ((obj), KZ_TYPE_EXT, KzExt))
-#define KZ_EXT_CLASS(klass)		(G_TYPE_CHECK_CLASS_CAST ((klass), KZ_TYPE_EXT, KzExtClass))
-#define KZ_IS_EXT(obj)			(G_TYPE_CHECK_INSTANCE_TYPE ((obj), KZ_TYPE_EXT))
-#define KZ_IS_EXT_CLASS(klass)		(G_TYPE_CHECK_CLASS_TYPE ((klass), KZ_TYPE_EXT))
-#define KZ_EXT_GET_CLASS(obj)		(G_TYPE_INSTANCE_GET_CLASS ((obj), KZ_TYPE_EXT, KzExtClass))
-
-typedef struct _KzExt		KzExt;
-typedef struct _KzExtClass	KzExtClass;
-
-typedef void (*KzExtInitFunc)  (KzWindow *window);
-typedef void (*KzExtCloseFunc) (gboolean success);
-
-struct _KzExt
-{
-	GObject parent;
-
-	gchar *name;
-	gchar *path;
-	GModule *module;
-};
-
-struct _KzExtClass
-{
-	GObjectClass parent_class;
-
-	/* -- signals -- */
-};
-
-GType	 kz_ext_get_type	(void) G_GNUC_CONST;
-GList	*kz_ext_load		(KzWindow *kz);
+typedef void (*KzExtInitFunc)  (void);
+typedef void (*KzExtSetupFunc) (KzWindow *window);
+typedef void (*KzExtExitFunc)  (gboolean success);
 
+void	 kz_ext_init		(void);
+void	 kz_ext_setup		(KzWindow *kz);
+void	 kz_ext_exit		(void);
 
 G_END_DECLS
 
