@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-xml.c,v 1.24 2004/11/22 01:08:41 ikezoe Exp $
+ *  $Id: kz-xml.c,v 1.25 2005/05/09 01:20:12 ikezoe Exp $
  */
 
 #include "kz-xml.h"
@@ -27,10 +27,12 @@
 #include <ctype.h>
 #include "gobject-utils.h"
 
-struct _KzXMLPriv
+typedef struct _KzXMLPrivate    KzXMLPrivate;
+struct _KzXMLPrivate
 {
 	GMarkupParseContext *context;
 };
+#define KZ_XML_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), KZ_TYPE_XML, KzXMLPrivate))
 
 static void kz_xml_class_init   (KzXMLClass *klass);
 static void kz_xml_init         (KzXML      *xml);
@@ -44,7 +46,6 @@ static GObjectClass *parent_class = NULL;
 KZ_OBJECT_GET_TYPE(kz_xml, "KzXML", KzXML,
 		   kz_xml_class_init, kz_xml_init,
 		   G_TYPE_OBJECT)
-KZ_OBJECT_FINALIZE(kz_xml, KzXML)
 
 
 static void
@@ -56,19 +57,20 @@ kz_xml_class_init (KzXMLClass *klass)
 	object_class = (GObjectClass *) klass;
 
 	object_class->dispose  = kz_xml_dispose;
-	object_class->finalize = kz_xml_finalize;
+	g_type_class_add_private (object_class, sizeof(KzXMLPrivate));
 }
 
 
 static void
 kz_xml_init (KzXML *xml)
 {
-	xml->file          = NULL;
-	xml->dtd           = NULL;
-	xml->encoding      = g_strdup("UTF-8");
-	xml->root          = kz_xml_node_new(xml, KZ_XML_NODE_DOC_ROOT);
-	xml->priv          = g_new0(KzXMLPriv, 1);
-	xml->priv->context = NULL;
+	KzXMLPrivate *priv = KZ_XML_GET_PRIVATE (xml);
+
+	xml->file     = NULL;
+	xml->dtd      = NULL;
+	xml->encoding = g_strdup("UTF-8");
+	xml->root     = kz_xml_node_new(xml, KZ_XML_NODE_DOC_ROOT);
+	priv->context = NULL;
 }
 
 
@@ -252,11 +254,13 @@ kz_xml_load_xml (KzXML *xml, const gchar *buffer, guint length)
 	ParseContext *ctx;
 	gboolean ret = FALSE;
 	GError *error = NULL; 
+	KzXMLPrivate *priv;
 
 	g_return_val_if_fail(KZ_IS_XML(xml), FALSE);
 	if (buffer == NULL) return FALSE;
 
-	context = xml->priv->context;
+	priv = KZ_XML_GET_PRIVATE (xml);
+	context = priv->context;
 	if (!context)
 	{
 		kz_xml_clear_content(xml);
@@ -268,7 +272,7 @@ kz_xml_load_xml (KzXML *xml, const gchar *buffer, guint length)
 		context = g_markup_parse_context_new
 				(&parser, 0,
 				 ctx, (GDestroyNotify) g_free);
-		xml->priv->context = context;
+		priv->context = context;
 	}
 
 	if (g_markup_parse_context_parse(context, buffer, length, &error))
@@ -282,7 +286,7 @@ kz_xml_load_xml (KzXML *xml, const gchar *buffer, guint length)
 		g_error_free(error);
 	}
 	g_markup_parse_context_free (context);
-	xml->priv->context = NULL;
+	priv->context = NULL;
 
 	return ret;
 }
