@@ -1319,7 +1319,7 @@ KzMozWrapper::GetFileToLocal (const nsACString &URI,
 		gchar *filename = create_filename_from_uri(uri);
 
 		dir = g_build_filename(g_get_home_dir(),
-				       ".", PACKAGE,
+				       "."PACKAGE,
 				       storedir,
 				       type, /* images or css */ 
 				       filename,
@@ -1552,6 +1552,62 @@ KzMozWrapper::SetImageZoom (float aZoom)
 	return NS_OK;
 }
 
+nsresult
+KzMozWrapper::GetDestAnchorsFromWindow (nsIDOMWindow *domWindow,
+					GList **list)
+{
+	nsresult rv;
+	gboolean flag = FALSE;
+	const PRUnichar aLiteral[] = { 'a', '\0' };
+
+	nsCOMPtr<nsIDOMDocument> domDoc;
+	domWindow->GetDocument(getter_AddRefs(domDoc));
+
+	nsCOMPtr<nsIDOMNodeList> nodeList;
+	rv = domDoc->GetElementsByTagName(nsEmbedString(aLiteral),
+			getter_AddRefs(nodeList));
+	if (NS_FAILED(rv) || !nodeList) return NS_ERROR_FAILURE;
+
+	PRUint32 num;
+	rv = nodeList->GetLength(&num);
+	if (NS_FAILED(rv) || num < 1) return NS_ERROR_FAILURE;
+
+	nsEmbedCString docURL;
+	GetDocumentUrl(docURL);
+
+	// store links to GList
+	nsCOMPtr<nsIDOMNode> node;
+	for (PRUint32 i = 0; i < num; i++)
+	{
+		rv = nodeList->Item(i, getter_AddRefs(node));
+		if (NS_FAILED(rv) || !node) continue;
+
+		// check the node has name attribute or id attribure.
+		char *value =  NULL;
+		GetAttributeFromNode(node, "id", &value);
+		if (!value)
+			GetAttributeFromNode(node, "name", &value);
+		if (!value)
+			continue;
+
+		nsEmbedCString cValue, cURI;
+		cValue.Assign("#");
+		cValue.Append(value, strlen(value));
+
+		ResolveURI(domDoc, cValue, cURI);	
+		g_free(value);
+
+		KzBookmark *link;
+		link = kz_bookmark_new_with_attrs(NULL, cURI.get(), NULL);
+		*list = g_list_append(*list, link);
+
+		flag = TRUE;
+	}
+
+	return flag ? NS_OK : NS_ERROR_FAILURE;
+}
+
+
 nsresult
 KzMozWrapper::GetLinksFromWindow (nsIDOMWindow *domWindow,
 				  GList **list,
@@ -1584,7 +1640,7 @@ KzMozWrapper::GetLinksFromWindow (nsIDOMWindow *domWindow,
 				flag |= TRUE;
 		}
 	}
-	else
+
 	{
 		nsCOMPtr<nsIDOMDocument> domDoc;
 		domWindow->GetDocument(getter_AddRefs(domDoc));
@@ -1646,7 +1702,6 @@ KzMozWrapper::GetAttributeFromNode (nsIDOMNode *node, const char *attr, char **v
 	if (!NS_SUCCEEDED(result) || !attrNode)  return NS_ERROR_FAILURE;
 
 	nsEmbedString nodeValue;
-
 	result = attrNode->GetNodeValue(nodeValue);
 	if (!NS_SUCCEEDED(result))  return NS_ERROR_FAILURE;
 
@@ -1654,6 +1709,7 @@ KzMozWrapper::GetAttributeFromNode (nsIDOMNode *node, const char *attr, char **v
 	NS_UTF16ToCString(nodeValue,
 			  NS_CSTRING_ENCODING_UTF8, cNodeValue);
 	*value = g_strdup(cNodeValue.get());
+
 	return NS_OK;
 }
 
@@ -1666,26 +1722,12 @@ KzMozWrapper::GetLinkFromNode (nsIDOMDocument *domDoc, nsIDOMNode *node,
 	// get url
 	char *hrefattr =  NULL;
 	GetAttributeFromNode(node, "href", &hrefattr);
-	if (!hrefattr) return NS_ERROR_FAILURE;	
-
+	if (!hrefattr) return NS_ERROR_FAILURE;
 
 	nsEmbedCString hrefc,linkc;
 	hrefc.Assign(hrefattr);
 
-	nsCOMPtr<nsIDOM3Node> domnode = do_QueryInterface(domDoc);
-     	if(!domnode) return NS_ERROR_FAILURE;
-	
-	nsEmbedString spec;
-	domnode->GetBaseURI(spec);
-
-	nsEmbedCString cSpec;
-	NS_UTF16ToCString(spec,
-			  NS_CSTRING_ENCODING_UTF8, cSpec);
-
-	nsCOMPtr<nsIURI> baseURI;
-	NewURI(getter_AddRefs(baseURI), cSpec.get());
-	
-	nsresult rv = baseURI->Resolve(hrefc,linkc);
+	ResolveURI(domDoc, hrefc, linkc);
 
 	*url = g_strdup(linkc.get());
 
