@@ -36,6 +36,8 @@
 #include "kz-mozselectionlistener.h"
 #include "kz-mozeventlistener.h"
 
+class nsISecureBrowserUI;
+class nsISSLStatus;
 
 class KzMozWrapper
 {
@@ -125,6 +127,8 @@ public:
 				          GList **list,
 				          nsISelection *selection, 
 				          gboolean selected_only);
+	nsresult GetDestAnchorsFromWindow (nsIDOMWindow *domWindow,
+					   GList **list);
 	nsresult GetPostData             (nsIHistoryEntry *he,
 				          nsAString &postData);
 	nsresult SetPostData             (nsIHistoryEntry *he,
@@ -141,6 +145,8 @@ public:
 	    				  nsIDOMRange **startRange,
 	    				  nsIDOMRange **endRange);
 
+	nsresult ShowPageCertificate     (void);
+
 	nsCOMPtr<nsIWebBrowser>    mWebBrowser;
 protected:
 	nsCOMPtr<nsIDOMEventTarget>   mEventTarget;
@@ -149,6 +155,7 @@ protected:
 	KzMozEventListener           *mKzMozEventListener;
 	KzMozSelectionListener       *mKzMozSelectionListener;
 	PRPackedBool                  mSelectionWasCollapsed;
+	nsCOMPtr<nsISecureBrowserUI> mSecurityInfo;
 
 	nsresult GetListener         (void);
 	nsresult AttachListeners     (void);
@@ -182,4 +189,5 @@ protected:
 				     const gchar *storedir,
 				     const gchar *type,
 				     nsAString &LocalFile);
+	nsresult  GetSSLStatus      (nsISSLStatus **aSSLStatus);
 };
