@@ -3,7 +3,7 @@
 /*
  *  Copyright (C) 2003 Hiroyuki Ikezoe
  *  Copyright (C) 2003 Takuro Ashie
- *  Copyright (C) 2004 Hidetaka Iwai
+ *  Copyright (C) 2004-2005 Hidetaka Iwai
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -32,7 +32,7 @@
 #include <unistd.h>
 
 #ifdef USE_SSL
-#include <openssl/ssl.h>
+#include <gnutls/gnutls.h>
 #endif
 
 #include "gobject-utils.h"
@@ -42,7 +42,7 @@
 #include "kz-profile.h"
 #include "kz-proxy-item.h"
 #include "kz-prompt-dialog.h"
-#include "base64.h"
+#include "kz-base64.h"
 #include "eggmd5.h"
 
 #define BUFFER_SIZE 256
@@ -71,10 +71,9 @@ static guint n_methods = G_N_ELEMENTS(methods);
 #ifdef USE_SSL
 typedef struct _KzSSL
 {
-	SSL      *ssl;
-	SSL_CTX  *ctx;
+	gnutls_session_t session;
+	gnutls_certificate_credentials_t xcred;
 } KzSSL;
-
 #endif
 
 typedef struct _AuthParam
@@ -257,6 +256,7 @@ kz_http_class_init (KzHTTPClass *klass)
 			_("The Path of the URI"),
 			NULL,
 			G_PARAM_READWRITE));
+
 	g_type_class_add_private (object_class, sizeof(KzHTTPPrivate));
 }
 
@@ -454,7 +454,7 @@ kz_http_new (const gchar *uri)
 			   NULL);
 	
 #if USE_SSL
-	if (strncmp(guri->scheme, "https", 5) == 0)
+	if (guri && strncmp(guri->scheme, "https", 5) == 0)
 	{
 		KzHTTPPrivate *priv = KZ_HTTP_GET_PRIVATE (http);
 		priv->kz_ssl = g_new0(KzSSL, 1);
@@ -654,20 +654,28 @@ kz_http_read_chars (KzHTTP *http, GIOChannel *iochannel,
 		while (len > 0 && f)
 		{
 			int ret;
-			ret = SSL_read(priv->kz_ssl->ssl, pos, len);
+			ret = gnutls_record_recv (priv->kz_ssl->session, pos, len);
 
-			switch (SSL_get_error(priv->kz_ssl->ssl, ret))
+			if (ret > 0)
 			{
-			case SSL_ERROR_NONE:
 				pos += ret;
 				len -= ret;
-				break;
-			case SSL_ERROR_WANT_READ:
-			case SSL_ERROR_WANT_WRITE:
-				break;
-			default:
+			}
+			else if (ret == 0)
+			{
+                        /* EOF */
+				f = FALSE;
+
+			}
+			else if ((ret == GNUTLS_E_INTERRUPTED) ||
+				 (ret == GNUTLS_E_AGAIN))
+			{
+                        /* read again */
+				continue;
+			}
+			else {
+				gnutls_perror (ret);
 				f = FALSE;
-				break;
 			}
 		}
 		*bytes_read = count - len;
@@ -715,8 +723,7 @@ kz_http_read_line (KzHTTP *http, GIOChannel *iochannel,
 		tmp = g_string_new(NULL);
 		while (TRUE)
 		{
-			l = SSL_read(priv->kz_ssl->ssl, &c, 1);	
-
+			l = gnutls_record_recv (priv->kz_ssl->session, &c, 1);
 			if (l > 0)
 			{
 				tmp = g_string_append_c(tmp, c);
@@ -730,16 +737,14 @@ kz_http_read_line (KzHTTP *http, GIOChannel *iochannel,
 			{
 				break;
 			}
+			else if ((l == GNUTLS_E_INTERRUPTED) ||
+				 (l == GNUTLS_E_AGAIN))
+			{
+				continue;
+			}
 			else
 			{
-				int e;
-				e = SSL_get_error(priv->kz_ssl->ssl, l);
-				
-				if (e == SSL_ERROR_WANT_READ ||
-				    e == SSL_ERROR_NONE)
-				{
-					continue;
-				}
+				gnutls_perror (l);
 				iostatus = G_IO_STATUS_ERROR;
 				break;
 			}
@@ -1158,18 +1163,22 @@ cb_http_connect(GTcpSocket *socket,
 		while (len > 0)
 		{
 			int ret;
-			ret = SSL_write(priv->kz_ssl->ssl, pos, len);
+			ret = gnutls_record_send (priv->kz_ssl->session, pos, len);
 
-			switch (SSL_get_error(priv->kz_ssl->ssl, ret))
+			if(ret > 0)
 			{
-			case SSL_ERROR_NONE:
-				pos += ret;	
+				pos += ret;
 				len -= ret;
-				break;
-			case SSL_ERROR_WANT_WRITE:
-			case SSL_ERROR_WANT_READ:
-				break;
-			default:
+			}
+			else if ((ret == GNUTLS_E_INTERRUPTED) ||
+				 (ret == GNUTLS_E_AGAIN))
+			{
+                                /* write again */
+				continue;
+			}
+			else
+			{
+				gnutls_perror (ret);
 				break;
 			}
 		}
@@ -1361,14 +1370,18 @@ kz_http_show_dialog (KzHTTP *http)
 
 	kz_prompt_dialog_set_title(prompt, _("Authentication"));
 #ifdef USE_SSL
-	if (priv->kz_ssl) {
+	if (priv->kz_ssl)
+	{
 		scheme = "https";
 		default_port = 443;
 	}
 #endif
-	if (priv->port == default_port) {
+	if (priv->port == default_port)
+	{
 		message = g_strdup_printf (_("Enter username and password for \"%s\" at %s://%s"), ap->realm, scheme, priv->hostname);
-	} else {
+	}
+	else
+	{
 		message = g_strdup_printf (_("Enter username and password for \"%s\" at %s://%s:%u"), ap->realm, scheme, priv->hostname, priv->port);
 	}
 	kz_prompt_dialog_set_message_text(prompt, message);
@@ -1388,7 +1401,7 @@ kz_http_show_dialog (KzHTTP *http)
 		 {
 			gchar *str, *base64;
 			str = g_strdup_printf("%s:%s", user, pass);
-			base64 = base64_encode(str);
+			base64 = kz_base64_encode(str);
 			ap->string = g_strdup_printf("Basic %s", base64);
 			g_free(str);
 			if (base64)
@@ -1479,51 +1492,44 @@ kz_http_show_dialog (KzHTTP *http)
 }
 
 #ifdef USE_SSL
+
 static void
 ssl_init (KzSSL *kz_ssl, GIOChannel *channel)
 {
 	gint fd;
-	int ret;
-	gboolean f = TRUE;
-	char *str;
-	BIO *bio;
-	X509 *server_cert;
+	gint ret;
+	const int cert_type_priority[] = { GNUTLS_CRT_X509,
+					   0 };
 
-	SSL_library_init();
-	OpenSSL_add_all_algorithms();
-	SSL_load_error_strings();
-
-	kz_ssl->ctx = SSL_CTX_new(SSLv23_client_method());
-
-	/*
-	SSL_CTX_load_verify_locations(kz_ssl->ctx, NULL, "/home/zoe/.kazehakase");
-	*/
-	kz_ssl->ssl = SSL_new(kz_ssl->ctx);
-	if (!kz_ssl->ssl)
-		return;
-
-	SSL_CTX_set_mode(kz_ssl->ctx, SSL_MODE_AUTO_RETRY | SSL_MODE_ACCEPT_MOVING_WRITE_BUFFER);
-
-	fd = g_io_channel_unix_get_fd(channel);
-	bio = BIO_new_socket(fd, BIO_NOCLOSE);
-	SSL_set_fd(kz_ssl->ssl, fd);
-	SSL_set_bio(kz_ssl->ssl, bio, bio);
-	BIO_set_nbio(bio, 1);
-
-	while (f)
+	ret = gnutls_certificate_allocate_credentials (&kz_ssl->xcred);
+	if(ret < 0)
 	{
-		ret = SSL_connect(kz_ssl->ssl);
-		switch (SSL_get_error(kz_ssl->ssl, ret))
-		{
-		case SSL_ERROR_WANT_READ:
-			continue;
-			break;
-		default:
-			f = FALSE;
-			break;
-		}
+		g_warning("gnutls_certificate_allocate_credentials: %s", gnutls_strerror(ret));
+		return;
 	}
-				
+        /* initialize TLS session */
+	gnutls_init (&kz_ssl->session, GNUTLS_CLIENT);
+
+	gnutls_set_default_priority (kz_ssl->session);
+	gnutls_certificate_type_set_priority (kz_ssl->session, cert_type_priority);
+
+	gnutls_credentials_set (kz_ssl->session, GNUTLS_CRD_CERTIFICATE, kz_ssl->xcred);
+	fd = g_io_channel_unix_get_fd(channel);
+	gnutls_transport_set_ptr (kz_ssl->session,  (gnutls_transport_ptr)fd);
+
+	do
+	{
+		ret = gnutls_handshake (kz_ssl->session);
+	}
+	while ((ret == GNUTLS_E_AGAIN) || (ret == GNUTLS_E_INTERRUPTED));
+
+	if (ret < 0)
+	{
+		gnutls_perror(ret);
+		return;
+	}
+
+#if 0
 	server_cert = SSL_get_peer_certificate(kz_ssl->ssl);
 
 	if (!server_cert)
@@ -1544,6 +1550,7 @@ ssl_init (KzSSL *kz_ssl, GIOChannel *channel)
 	{
 	}
 	*/
+#endif
 }
 
 static void
@@ -1551,10 +1558,9 @@ ssl_term (KzSSL *kz_ssl)
 {
 	if (!kz_ssl)
 		return;
-
-	SSL_shutdown(kz_ssl->ssl);
-	SSL_free(kz_ssl->ssl);
-	SSL_CTX_free(kz_ssl->ctx);
+	gnutls_bye (kz_ssl->session, GNUTLS_SHUT_RDWR);
+	gnutls_certificate_free_credentials (kz_ssl->xcred);
+	gnutls_deinit (kz_ssl->session);
 }
 
 #endif
