@@ -793,9 +793,9 @@ kz_io_decode_string(GString *string)
 	z.zalloc = Z_NULL;
 	z.zfree = Z_NULL;
 	z.opaque = Z_NULL;
-	z.next_in = string->str;
+	z.next_in = (unsigned char*)string->str;
 	z.avail_in = string->len;
-	z.next_out = outbuf;
+	z.next_out = (unsigned char*)outbuf;
 	z.avail_out = OUTBUFSIZ;
 
 	/* windowBits+32 means to enable zlib and gzip decoding
@@ -814,7 +814,7 @@ kz_io_decode_string(GString *string)
 		if (status == Z_STREAM_END || status == Z_OK)
 		{
 			g_string_append_len(outstring, outbuf, OUTBUFSIZ - z.avail_out);
-			z.next_out = outbuf;
+			z.next_out = (unsigned char*)outbuf;
 			z.avail_out = OUTBUFSIZ;
 		}
 		else
