@@ -468,7 +468,7 @@ prefs_new_tab_create (void)
 			 G_CALLBACK(cb_new_tab_changed), prefsui);
 	gtk_widget_show(check);
 
-	check = gtk_check_button_new_with_mnemonic(_("Move _Keyboad focus to location entry when swich to new empty tab"));
+	check = gtk_check_button_new_with_mnemonic(_("Move _Keyboard focus to location entry when switch to new empty tab"));
 	prefsui->focus_loc_ent = check;
 	gtk_box_pack_start(GTK_BOX(vbox), check, FALSE, FALSE, 2);
 	key_focus = FALSE;
