@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-downloader-sidebar.c,v 1.10 2005/07/10 08:53:37 ikezoe Exp $
+ *  $Id: kz-downloader-sidebar.c,v 1.11 2006/05/16 00:18:52 ikezoe Exp $
  */
 
 #include "kz-downloader-sidebar.h"
@@ -25,11 +25,6 @@
 #include <glib/gi18n.h>
 #include "kazehakase.h"
 #include "gobject-utils.h"
-#if GTK_CHECK_VERSION(2,6,0)
-#define kz_cell_renderer_progress_new gtk_cell_renderer_progress_new
-#else
-#include "kz-cellrendererprogress.h"
-#endif
 
 enum {
 	PROP_0,
@@ -282,7 +277,7 @@ kz_downloader_sidebar_init (KzDownloaderSidebar *sidebar)
 	gtk_container_add(GTK_CONTAINER(scrwin), GTK_WIDGET(tree_view));
 	gtk_widget_show(widget);
 
-	cell = kz_cell_renderer_progress_new();
+	cell = gtk_cell_renderer_progress_new();
 	column = gtk_tree_view_column_new_with_attributes
 			(_("Progress"), cell,
 			 "percentage", COLUMN_PROGRESS,
