@@ -26,9 +26,7 @@
 #include "kz-tab-label.h"
 #include "kz-actions-tab.h"
 
-#define NORMAL_COLOR  NULL
-#define LOADING_COLOR "red"
-
+static gchar *label_color[KZ_TAB_LABEL_N_STATE];
 
 enum {
 	PROP_0,
@@ -98,6 +96,10 @@ static void     cb_append_tab     (KzWindow        *kz,
 				   GtkWidget       *widget,
 				   GtkWidget       *parent,
 				   KzTabTree       *tabtree);
+static void     parent_append     (KzTabTree * tabtree, 
+				   KzEmbed * kzembed, 
+				   GtkTreeIter * parent, 
+				   GtkTreeIter * iter);
 static void     cb_remove_tab     (KzWindow        *kz,
 				   GtkWidget       *widget,
 				   KzTabTree       *tabtree);
@@ -223,6 +225,34 @@ kz_tab_tree_init (KzTabTree *tabtree)
 	GtkCellRenderer *cell;
 	GtkTreeViewColumn *column;
 
+	gchar *color;
+
+        /* label color */
+        color = KZ_CONF_GET_STR("Tab" , "normal_color");
+        if (!color)
+                color = g_strdup("#000000");
+
+        if (label_color[KZ_TAB_LABEL_STATE_NORMAL])
+                g_free(label_color[KZ_TAB_LABEL_STATE_NORMAL]);
+        label_color[KZ_TAB_LABEL_STATE_NORMAL] = g_strdup(color);
+        g_free(color);
+
+        color = KZ_CONF_GET_STR("Tab" , "loading_color");
+        if (!color)
+                color = g_strdup("#ff0000");
+        if (label_color[KZ_TAB_LABEL_STATE_LOADING])
+                g_free(label_color[KZ_TAB_LABEL_STATE_LOADING]);
+        label_color[KZ_TAB_LABEL_STATE_LOADING] = g_strdup(color);
+        g_free(color);
+
+        color = KZ_CONF_GET_STR("Tab" , "loaded_color");
+        if (!color)
+                color = g_strdup("#22aa44");
+        if (label_color[KZ_TAB_LABEL_STATE_LOADED])
+                g_free(label_color[KZ_TAB_LABEL_STATE_LOADED]);
+        label_color[KZ_TAB_LABEL_STATE_LOADED] = g_strdup(color);
+        g_free(color);
+
 	scrwin = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrwin),
 				       GTK_POLICY_AUTOMATIC,
@@ -474,36 +504,7 @@ build_tab_tree (KzTabTree *tabtree, GNode *parent)
 
 		if (KZ_IS_EMBED(kzembed))
 		{
-			gchar *title = kz_embed_ensure_title(kzembed);
-			const gchar *uri   = kz_embed_get_location(kzembed);
-			GdkPixbuf *favicon = kz_favicon_get_pixbuf(tabtree->sidebar->kz->kzfav,
-								   uri,
-								   GTK_ICON_SIZE_MENU);
-			const gchar *color;
-			color = kz_embed_is_loading(kzembed)
-				? LOADING_COLOR : NORMAL_COLOR;
-
-			gtk_tree_store_append(tabtree->store,
-					      &iter, parent_iter);
-			gtk_tree_store_set(tabtree->store, &iter,
-					   COLUMN_ICON,        favicon,
-					   COLUMN_TITLE,       title,
-					   COLUMN_TITLE_COLOR, color,
-					   COLUMN_EMBEDER,     kzembed,
-					   COLUMN_TERMINATOR);
-			g_signal_connect(kzembed, "kz-title",
-					 G_CALLBACK(cb_title_changed),
-					 tabtree);
-			g_signal_connect(kzembed, "kz-net-start",
-					 G_CALLBACK(cb_net_start),
-					 tabtree);
-			g_signal_connect(kzembed, "kz-net-stop",
-					 G_CALLBACK(cb_net_stop),
-					 tabtree);
-
-			g_free(title);
-			if (favicon)
-				g_object_unref(favicon);
+			parent_append(tabtree, kzembed, parent_iter, &iter);
 		}
 		else
 		{
@@ -541,35 +542,7 @@ build_tab_list (KzTabTree *tabtree)
 
 		if (KZ_IS_EMBED(kzembed))
 		{
-			gchar *title = kz_embed_ensure_title(kzembed);
-			const gchar *color;
-			const gchar *uri   = kz_embed_get_location(kzembed);
-			GdkPixbuf *favicon = kz_favicon_get_pixbuf(tabtree->sidebar->kz->kzfav,
-								   uri,
-								   GTK_ICON_SIZE_MENU);
-			color = kz_embed_is_loading(kzembed)
-				? LOADING_COLOR : NORMAL_COLOR;
-
-			gtk_tree_store_append(tabtree->store, &iter, NULL);
-			gtk_tree_store_set(tabtree->store, &iter,
-					   COLUMN_ICON,        favicon,
-					   COLUMN_TITLE,       title,
-					   COLUMN_TITLE_COLOR, color,
-					   COLUMN_EMBEDER,     kzembed,
-					   COLUMN_TERMINATOR);
-			g_signal_connect(kzembed, "kz-title",
-					 G_CALLBACK(cb_title_changed),
-					 tabtree);
-			g_signal_connect(kzembed, "kz-net-start",
-					 G_CALLBACK(cb_net_start),
-					 tabtree);
-			g_signal_connect(kzembed, "kz-net-stop",
-					 G_CALLBACK(cb_net_stop),
-					 tabtree);
-
-			g_free(title);
-			if (favicon)
-				g_object_unref(favicon);
+			parent_append(tabtree, kzembed, NULL, &iter);
 		}
 		else
 		{
@@ -783,6 +756,7 @@ cb_switch_page (GtkNotebook *notebook, GtkNotebookPage *page,
 	GtkWidget *widget;
 	GtkTreeIter *iter;
 	GtkTreePath *treepath;
+	KzTabLabel *label;
 
 	g_return_if_fail(KZ_IS_TAB_TREE(tabtree));
 
@@ -799,6 +773,19 @@ cb_switch_page (GtkNotebook *notebook, GtkNotebookPage *page,
 
 	tabtree->switching = TRUE;
 	gtk_tree_view_set_cursor(tabtree->tree_view, treepath, NULL, FALSE);
+
+	label = KZ_TAB_LABEL(gtk_notebook_get_tab_label(notebook, widget));
+	
+	gtk_tree_store_set(tabtree->store, iter,
+			   COLUMN_TITLE,   kz_embed_ensure_title(KZ_EMBED(widget)),
+			   COLUMN_TITLE_COLOR, label_color[label->state],
+			   COLUMN_TERMINATOR);
+
+	if (!gtk_tree_view_row_expanded(tabtree->tree_view, treepath))
+		gtk_tree_path_up(treepath);
+		
+        gtk_tree_view_expand_to_path(tabtree->tree_view, treepath);
+	
 	tabtree->switching = FALSE;
 
 	gtk_tree_path_free(treepath);
@@ -810,55 +797,115 @@ static void
 cb_append_tab (KzWindow *kz, GtkWidget *widget, GtkWidget *parent,
 	       KzTabTree *tabtree)
 {
-	GNode *tree;
+	GtkTreeIter iter;
+	KzEmbed * kzembed = KZ_EMBED(widget); 
 
 	g_return_if_fail(GTK_IS_WIDGET(widget));
 	g_return_if_fail(KZ_IS_TAB_TREE(tabtree));
 
-	tree = kz_window_get_tree(tabtree->sidebar->kz);
-
-	/* FIXME!! It's a quick hack */
-	remove_all_embed_signals(tabtree);
-	gtk_tree_store_clear(tabtree->store);
-
-	if (tabtree->tree)
-	{
-		if (tree && g_node_first_child(tree))
-			build_tab_tree(tabtree, tree);
-		gtk_tree_view_expand_all(tabtree->tree_view);
-	}
-	else
-	{
-		build_tab_list(tabtree);
-	}
+	parent_append(tabtree, kzembed, find_node(tabtree->store, KZ_EMBED(parent)), &iter);
+	
+	GtkTreePath * path = gtk_tree_model_get_path(GTK_TREE_MODEL(tabtree->store), &iter);
+	gtk_tree_view_expand_to_path(tabtree->tree_view, path);
+	gtk_tree_path_free(path);
 }
 
 
+static void 
+parent_append(KzTabTree * tabtree, KzEmbed * kzembed, GtkTreeIter * parent, GtkTreeIter * iter) {
+
+	const gchar *color;
+	gchar *title = kz_embed_ensure_title(kzembed);
+	const gchar *uri   = kz_embed_get_location(kzembed);
+	GdkPixbuf *favicon = kz_favicon_get_pixbuf(tabtree->sidebar->kz->kzfav,
+						   uri,
+						   GTK_ICON_SIZE_MENU);
+
+	KzTabLabel * label = KZ_TAB_LABEL(gtk_notebook_get_tab_label(GTK_NOTEBOOK(tabtree->sidebar->kz->notebook), GTK_WIDGET(kzembed)));
+	color = label_color[label->state];
+
+	gtk_tree_store_append(tabtree->store, iter, parent);
+	gtk_tree_store_set(tabtree->store, iter,
+			   COLUMN_ICON,        favicon,
+			   COLUMN_TITLE,       title,
+			   COLUMN_TITLE_COLOR, color,
+			   COLUMN_EMBEDER,     kzembed,
+			   COLUMN_TERMINATOR);
+
+	g_signal_connect(kzembed, "kz-title",
+			 G_CALLBACK(cb_title_changed),
+			 tabtree);
+	g_signal_connect(kzembed, "kz-net-start",
+			 G_CALLBACK(cb_net_start),
+			 tabtree);
+	g_signal_connect(kzembed, "kz-net-stop",
+			 G_CALLBACK(cb_net_stop),
+			 tabtree);
+
+	g_free(title);
+	if (favicon)
+		g_object_unref(favicon);
+}
+
+
+static void
+move_to(KzTabTree * tabtree, GtkTreeIter * child, GtkTreeIter * parent)
+{
+	KzEmbed * kzembed;
+	GtkTreeIter newparent, newchild;
+	GtkTreePath *path;
+	
+        g_return_if_fail(KZ_IS_TAB_TREE(tabtree));
+	
+	gtk_tree_model_get(GTK_TREE_MODEL(tabtree->store), child, COLUMN_EMBEDER, &kzembed, COLUMN_TERMINATOR);
+	parent_append(tabtree, kzembed, parent, &newparent);
+
+	path = gtk_tree_model_get_path(GTK_TREE_MODEL(tabtree->store),
+                                       child);
+	if (gtk_tree_path_up(path) && gtk_tree_view_row_expanded(tabtree->tree_view, path))
+	{
+                GtkTreePath * path2 = gtk_tree_model_get_path(GTK_TREE_MODEL(tabtree->store),
+                                                              &newparent);
+                gtk_tree_view_expand_to_path(tabtree->tree_view, path2);
+                gtk_tree_path_free(path2);
+	}
+        gtk_tree_path_free(path);
+
+        while (gtk_tree_model_iter_children(GTK_TREE_MODEL(tabtree->store), &newchild, child))
+	{
+                move_to(tabtree, &newchild, &newparent);
+        }
+
+        gtk_tree_store_remove(tabtree->store, child);
+
+}
+
 static void
 cb_remove_tab (KzWindow *kz, GtkWidget *widget,
 	       KzTabTree *tabtree)
 {
-	GNode *tree;
+	GtkTreeIter child;
+	GtkTreeIter *parent;
 
 	g_return_if_fail(GTK_IS_WIDGET(widget));
 	g_return_if_fail(KZ_IS_TAB_TREE(tabtree));
 
-	tree = kz_window_get_tree(tabtree->sidebar->kz);
+	parent = find_node(tabtree->store, KZ_EMBED(widget));
 
-	/* FIXME!! It's a quick hack */
-	remove_all_embed_signals(tabtree);
-	gtk_tree_store_clear(tabtree->store);
-
-	if (tabtree->tree)
+	while (gtk_tree_model_iter_children(GTK_TREE_MODEL(tabtree->store), &child, parent))
 	{
-		if (tree && g_node_first_child(tree))
-			build_tab_tree(tabtree, tree);
-		gtk_tree_view_expand_all(tabtree->tree_view);
-	}
-	else
-	{
-		build_tab_list(tabtree);
+		GtkTreeIter grand;
+		if (gtk_tree_model_iter_parent(GTK_TREE_MODEL(tabtree->store), &grand, parent))
+		{
+			move_to(tabtree, &child, &grand);
+		}
+		else
+		{
+			move_to(tabtree, &child, NULL);
+		}
 	}
+	
+	gtk_tree_store_remove(tabtree->store, parent);
 }
 
 
@@ -899,7 +946,7 @@ cb_net_start (KzEmbed *embed, KzTabTree *tabtree)
 	{
 		gtk_tree_store_set(tabtree->store, iter,
 				   COLUMN_TITLE,       title,
-				   COLUMN_TITLE_COLOR, LOADING_COLOR,
+				   COLUMN_TITLE_COLOR, label_color[KZ_TAB_LABEL_STATE_LOADING],
 				   COLUMN_TERMINATOR);
 		gtk_tree_iter_free(iter);
 	}
@@ -921,10 +968,14 @@ cb_net_stop (KzEmbed *embed, KzTabTree *tabtree)
 	iter = find_node (tabtree->store, KZ_EMBED(embed));
 	if (iter)
 	{
+		KzTabLabel * label = KZ_TAB_LABEL(gtk_notebook_get_tab_label(GTK_NOTEBOOK(tabtree->sidebar->kz->notebook), GTK_WIDGET(embed)));
+
 		gtk_tree_store_set(tabtree->store, iter,
 				   COLUMN_TITLE,       title,
-				   COLUMN_TITLE_COLOR, NORMAL_COLOR,
+				   COLUMN_TITLE_COLOR,  
+				   label_color[label->state],
 				   COLUMN_TERMINATOR);
+
 		gtk_tree_iter_free(iter);
 	}
 
@@ -945,13 +996,13 @@ cb_title_changed (KzEmbed *embed, KzTabTree *tabtree)
 	iter = find_node (tabtree->store, KZ_EMBED(embed));
 	if (iter)
 	{
-		const gchar *color;
-		color = kz_embed_is_loading(KZ_EMBED(embed))
-			? LOADING_COLOR : NORMAL_COLOR;
+                KzTabLabel * label = KZ_TAB_LABEL(gtk_notebook_get_tab_label(GTK_NOTEBOOK(tabtree->sidebar->kz->notebook), GTK_WIDGET(embed)));
+
 		gtk_tree_store_set(tabtree->store, iter,
 				   COLUMN_TITLE, title,
-				   COLUMN_TITLE_COLOR, color,
+				   COLUMN_TITLE_COLOR, label_color[label->state],
 				   COLUMN_TERMINATOR);
+
 		gtk_tree_iter_free(iter);
 	}
 
